//-----------------------------------------------------------------------------
#include "Pch.h"
#include "Unicode.h"
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
//  Conversion methods
//-----------------------------------------------------------------------------

uint8 Unicode::UCS2toUTF8 ( uint16 _iUCS2, uint8 *_pUTF8 )
{
    return UCS4toUTF8 ( (uint32)_iUCS2, _pUTF8 ) ;
}

//-----------------------------------------------------------------------------

uint8 Unicode::UCS4toUTF8 ( uint32 _iUCS4, uint8 *_pUTF8 )
{
    if ( _iUCS4 < 0x00000080 )
    {
        _pUTF8[0] = uint8( _iUCS4 ) ;
        return 1 ;
    }
    else if ( _iUCS4 < 0x00000800 )
    {
        _pUTF8[0] = uint8( 0xc0 | ( _iUCS4 >>    6 ) ) ;
        _pUTF8[1] = uint8( 0x80 | ( _iUCS4 &  0x3f ) ) ;
        return 2 ;
    }
    else if ( _iUCS4 < 0x00010000 )
    {
        _pUTF8[0] = uint8( 0xe0 |   ( _iUCS4 >>   12 ) ) ;
        _pUTF8[1] = uint8( 0x80 | ( ( _iUCS4 >>    6 ) & 0x3f ) ) ;
        _pUTF8[2] = uint8( 0x80 |   ( _iUCS4 &  0x3f ) ) ;
        return 3 ;
    }
    else
    {
        _pUTF8[0] = uint8( 0xf0 |   ( _iUCS4 >>   18 ) ) ;
        _pUTF8[1] = uint8( 0x80 | ( ( _iUCS4 >>   12 ) & 0x3f ) ) ;
        _pUTF8[2] = uint8( 0x80 | ( ( _iUCS4 >>    6 ) & 0x3f ) ) ;
        _pUTF8[3] = uint8( 0x80 |   ( _iUCS4 &  0x3f ) ) ;
        return 4 ;
    }
}

//-----------------------------------------------------------------------------

uint8 Unicode::UTF8toUCS2 ( const uint8 *_pUTF8, uint16 *_pUCS2 )
{
    return 0 ; // TODO
}

//-----------------------------------------------------------------------------

uint8 Unicode::UTF8toUCS4 ( const uint8 *_pUTF8, uint32 *_pUCS4 )
{
	uint8   iReadBytes      = 0 ;
	uint8   iFirstOctetMask = 0 ;
	uint8   iFirstOctet     = _pUTF8[0] ;

	if      (   iFirstOctet < 0x80           ) { iReadBytes = 1 ; iFirstOctetMask = 0x7F ; }
    else if ( ( iFirstOctet & 0xE0 ) == 0xC0 ) { iReadBytes = 2 ; iFirstOctetMask = 0x1F ; } 
    else if ( ( iFirstOctet & 0xF0 ) == 0xE0 ) { iReadBytes = 3 ; iFirstOctetMask = 0x0F ; } 
    else if ( ( iFirstOctet & 0xF8 ) == 0xF0 ) { iReadBytes = 4 ; iFirstOctetMask = 0x07 ; }
    else if ( ( iFirstOctet & 0xFC ) == 0xF8 ) { iReadBytes = 5 ; iFirstOctetMask = 0x03 ; } 
    else if ( ( iFirstOctet & 0xFE ) == 0xFC ) { iReadBytes = 6 ; iFirstOctetMask = 0x01 ; } 
    else
    {
        *_pUCS4 = 0 ;
		return 0 ;  // The given chunk is not a valid UTF-8 encoded Unicode character
    }
	
	uint32 c = iFirstOctet & iFirstOctetMask ;
	
	for ( uint8 i = 1 ; i < iReadBytes ; i++ )
    {
		if ( ( _pUTF8[i] & 0xC0 ) != 0x80 ) 
        {
            *_pUCS4 = 0 ;
            return 0 ;
        }
		
		c <<= 6 ;
		c  |= _pUTF8[i] & 0x3F ;
	}

    *_pUCS4 = c ;

	return iReadBytes ;
}

//-----------------------------------------------------------------------------

uint16 Unicode::UTF8toUCS2 ( const uint8 *_pUTF8 )
{
    uint32          iUCS4 = 0 ;
    UTF8toUCS4   ( _pUTF8, &iUCS4 ) ;
    return uint16(  iUCS4 & 0x0000ffff ) ;
}

//-----------------------------------------------------------------------------

uint32 Unicode::UTF8toUCS4 ( const uint8 *_pUTF8 )
{
    uint32          iUCS4 = 0 ;
    UTF8toUCS4   ( _pUTF8, &iUCS4 ) ;
    return          iUCS4 ;
}

//-----------------------------------------------------------------------------
//  Utility methods
//-----------------------------------------------------------------------------

uint32 Unicode::UTF8StringCharCount ( const uint8 *_pUTF8 )
{
    uint32 iCount = 0 ;

    if ( _pUTF8 )
    {
        const uint8 *p = _pUTF8 ;
        while     ( *p )
        {
            // Any chars 0x80-0xBF are extension bytes.  
            // Anything else signals a new char.
            //
            uint8 c = *p++ ;

            if ( c < 0x80 || c > 0xBF )
            {
                iCount++ ;
            }
        }
    }
    return iCount ;
}
