﻿;
; ------------------------------------------------------------
;
;   PureBasic - polyglot loader
;
;    (c) 2015 - trisymphony.com
;
; ------------------------------------------------------------

; -- LOCALS --------------------------------------------------

Structure pg_langstruct
  langstring.s
  version.i
  location.s
EndStructure

Structure pg_dict
  Map dict.s()
EndStructure

Global NewMap availableLanguages.pg_langstruct()
Global NewMap languageDict.pg_dict()

Global pg_bInit = 0
#compound = 1




; -- WINDOWS DLL ---------------------------------------------

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  
  ; This procedure is called once, when the program loads the library
  ProcedureDLL AttachProcess(Instance)
  EndProcedure
  
  ; Called when the program releases (free) the DLL
  ProcedureDLL DetachProcess(Instance)
  EndProcedure
  
  ; Both are called when a thread in a program calls or releases (frees) the DLL
  ProcedureDLL AttachThread(Instance)
  EndProcedure
  
  ProcedureDLL DetachThread(Instance)
  EndProcedure
  
CompilerEndIf


; -- LIBRARY CODE --------------------------------------------

; init and load all available XMLs
ProcedureDLL pg_init(XMLFOLDER.s, SCHEME.s="Polyglot-*.xml", VERSION=0 ) ; version 0 means take most current version, otherwise take the specified version
  
  ; return codes for communication
  ; 0 - not initialized
  ; 1 - loaded and ready
  ; 2 - XML folder does not exist
  ; 3 - XML parsing error
  
  Static pXMLFOLDER.s
  Static pSCHEME.s
  ; cleanup input
  pXMLFOLDER = RTrim(XMLFOLDER, "/")
  pXMLFOLDER = RTrim(XMLFOLDER, "\")
  
  ; check for provided XML folder
  If FileSize(pXMLFOLDER) = -2
    
    ; scan directory for polyglot language xml files
    If ExamineDirectory(0, pXMLFOLDER, pSCHEME)
      While NextDirectoryEntry(0)
        If DirectoryEntryType(0) = #PB_DirectoryEntry_File
          
          ; disassemble file name
          FILENAME.s = DirectoryEntryName(0)
          ; get version info
          FILEVERSION = Val(Left(StringField(FILENAME, 2, "-"), 3))
          
          
          ; lang codes
          For k=2 To CountString(FILENAME, "_")+1
            NEWLANG.s = ReplaceString(StringField(FILENAME, k, "_"), ".xml", "")
            
            ; version check
            If FindMapElement(availableLanguages(), NEWLANG) ; entry already exists -> get version number
              CURRENTVERSION = availableLanguages(NEWLANG)\version
            Else ; not found -> uninitialized
              availableLanguages(NEWLANG)\version = 0
            EndIf
            
            If (VERSION = 0); compare versions and take the most recent one
              If (CURRENTVERSION < FILEVERSION)
                availableLanguages(NEWLANG)\langstring = NEWLANG
                availableLanguages(NEWLANG)\version = FILEVERSION ; for later comparison
                ; store file location
                CompilerIf #PB_Compiler_OS = #PB_OS_Windows
                  availableLanguages()\location = pXMLFOLDER + "\" + FILENAME
                CompilerElse
                  availableLanguages()\location = pXMLFOLDER + "/" + FILENAME
                CompilerEndIf
              EndIf
              
            ElseIf (VERSION = FILEVERSION) ; only if we specified a certain version number
              availableLanguages(NEWLANG)\langstring = NEWLANG
              availableLanguages(NEWLANG)\version = 0 ; we don't care
              ; store file location
              CompilerIf #PB_Compiler_OS = #PB_OS_Windows
                availableLanguages()\location = pXMLFOLDER + "\" + FILENAME
              CompilerElse
                availableLanguages()\location = pXMLFOLDER + "/" + FILENAME
              CompilerEndIf
            EndIf

          Next
          
        EndIf
      Wend
      FinishDirectory(0)
    EndIf ; end directory scan
    
    
    ; hack apart XML and store into global maps
    ForEach availableLanguages()
      If LoadXML(0, availableLanguages()\location )
        If XMLStatus(0) <> #PB_XML_Success
          ProcedureReturn 3
        EndIf
        
        *MainNode = MainXMLNode(0)
        If *MainNode
          ; Now get the first child node
          *ChildNode = ChildXMLNode(*MainNode)
          ; Loop through all available child nodes 
          While *ChildNode <> 0
            If GetXMLNodeName(*ChildNode) = "s"
              PGCODE.s = GetXMLAttribute(*ChildNode, "n")
              PGVAL.s = GetXMLNodeText(*ChildNode)
              languageDict(availableLanguages()\langstring)\dict(PGCODE) = PGVAL
            EndIf
            *ChildNode = NextXMLNode(*ChildNode)
          Wend ; end childnode
          
        EndIf ; end mainnode
      FreeXML(0)
      EndIf ; end XML load
      
    Next ; end availLang loop
    
    CreateRegularExpression(#compound, "[A-Z]+(_[A-Z]+)+")
    pg_bInit = 1
    ProcedureReturn 1
    
  Else
    ProcedureReturn 2
  EndIf ; end folder check
  
  ProcedureReturn 0
EndProcedure


; check if one particular language is loaded
ProcedureDLL pg_isLanguageAvailable(LANGCODE.s)
  
  Static pLANGCODE.s
  pLANGCODE = LANGCODE
  
  If pg_bInit = 1
    If FindMapElement(languageDict(), pLANGCODE)
      ProcedureReturn 1
    Else
      ProcedureReturn 0
    EndIf
  EndIf
  
  ProcedureReturn 0
EndProcedure


; translate string from code string
ProcedureDLL.s pg_translate(LANGCODE.s, CODESTRING.s, bLC=0)
  Static pLANGCODE.s
  Static pCODESTRING.s
  pLANGCODE = LANGCODE
  pCODESTRING = CODESTRING
  
  If pg_bInit = 1
    If FindMapElement(languageDict() ,pLANGCODE)
      If bLC = 0
        ProcedureReturn languageDict(pLANGCODE)\dict(pCODESTRING)
      Else
        ProcedureReturn LCase(languageDict(pLANGCODE)\dict(pCODESTRING))
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn ""
EndProcedure


; translate string that contains code strings among regular strings
ProcedureDLL.s pg_translateCompound(LANGCODE.s, CODESTRING.s, bLC=0)
  Static pg_returnTranslatedString.s
  Static pLANGCODE.s
  Static pCODESTRING.s
  pLANGCODE = LANGCODE
  pCODESTRING = CODESTRING
  pg_returnTranslatedString = pCODESTRING
  lcstr.s = ""
  
  If pg_bInit = 1
    
    ; regex
    If IsRegularExpression(#compound) And FindMapElement(languageDict() ,pLANGCODE)
      Dim Result$(0)
      NbFound = ExtractRegularExpression(#compound, pCODESTRING, Result$())
      For k = 0 To NbFound-1
        If FindMapElement(languageDict(pLANGCODE)\dict(), Result$(k))
          
          If bLC = 1
            lcstr = LCase(languageDict(pLANGCODE)\dict(Result$(k)))
          Else
            lcstr = languageDict(pLANGCODE)\dict(Result$(k))
          EndIf
          
          pg_returnTranslatedString = ReplaceString(pg_returnTranslatedString, Result$(k), lcstr)
        EndIf
      Next
    EndIf
  EndIf
  
  ProcedureReturn pg_returnTranslatedString
EndProcedure

; IDE Options = PureBasic 5.24 LTS (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 210
; FirstLine = 175
; Folding = --
; EnableUnicode
; EnableThread
; Executable = polyglot32.dll